<?php

namespace Bixo\Employee\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Bixo\Employee\Http\Requests\AttendanceRequest;
use Bixo\Employee\Interfaces\AttendanceRepositoryInterface;
use Bixo\Employee\Models\Attendance;
use Bixo\Employee\Forms\Attendance as Form;

/**
 * APIController  class for attendance.
 */
class AttendanceAPIController extends BaseController
{

    /**
     * Initialize attendance resource controller.
     *
     * @param type AttendanceRepositoryInterface $attendance
     *
     * @return null
     */
    public function __construct(AttendanceRepositoryInterface $attendance)
    {
        parent::__construct();
        $this->repository = $attendance;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Employee\Repositories\Criteria\AttendanceResourceCriteria::class);
    }

    /**
     * Display a list of attendance.
     *
     * @return Response
     */
    public function index(AttendanceRequest $request)
    {
        return $this->repository
            ->setPresenter(\Bixo\Employee\Repositories\Presenter\AttendancePresenter::class)
            ->paginate();
    }

    /**
     * Display attendance.
     *
     * @param Request $request
     * @param Model   $attendance
     *
     * @return Response
     */
    public function show(AttendanceRequest $request, Attendance $attendance)
    {
        return $attendance->setPresenter(\Bixo\Employee\Repositories\Presenter\AttendanceListPresenter::class);
        ;
    }

    /**
     * Create new attendance.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AttendanceRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('employee::attendance.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('employee/attendance/' . $attendance->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('employee/attendance');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the attendance.
     *
     * @param Request $request
     * @param Model   $attendance
     *
     * @return Response
     */
    public function update(AttendanceRequest $request, Attendance $attendance)
    {
        try {
            $data = $request->all();

            $attendance->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('employee::attendance.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('employee/attendance/' . $attendance->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('employee/attendance/' . $attendance->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the attendance.
     *
     * @param Model   $attendance
     *
     * @return Response
     */
    public function destroy(AttendanceRequest $request, Attendance $attendance)
    {
        try {
            $attendance->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('employee::attendance.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('employee/attendance/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('employee/attendance/' . $attendance->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
