<?php

namespace Bixo\Employee\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Employee\Http\Requests\AttendanceRequest;
use Bixo\Employee\Interfaces\AttendanceRepositoryInterface;
use Bixo\Employee\Models\Attendance;

/**
 * Resource controller class for attendance.
 */
class AttendanceResourceController extends BaseController
{

    /**
     * Initialize attendance resource controller.
     *
     * @param type AttendanceRepositoryInterface $attendance
     *
     * @return null
     */
    public function __construct(AttendanceRepositoryInterface $attendance)
    {
        parent::__construct();
        $this->repository = $attendance;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Employee\Repositories\Criteria\AttendanceResourceCriteria::class);
    }

    /**
     * Display a list of attendance.
     *
     * @return Response
     */
    public function index(AttendanceRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Employee\Repositories\Presenter\AttendancePresenter::class)
                ->$function();
        }

        $attendances = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('employee::attendance.names'))
            ->view('employee::attendance.index', true)
            ->data(compact('attendances', 'view'))
            ->output();
    }

    /**
     * Display attendance.
     *
     * @param Request $request
     * @param Model   $attendance
     *
     * @return Response
     */
    public function show(AttendanceRequest $request, Attendance $attendance)
    {

        if ($attendance->exists) {
            $view = 'employee::attendance.show';
        } else {
            $view = 'employee::attendance.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('employee::attendance.name'))
            ->data(compact('attendance'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new attendance.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AttendanceRequest $request)
    {

        $attendance = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('employee::attendance.name')) 
            ->view('employee::attendance.create', true) 
            ->data(compact('attendance'))
            ->output();
    }

    /**
     * Create new attendance.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AttendanceRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $attendance                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('employee::attendance.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('employee/attendance/' . $attendance->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/employee/attendance'))
                ->redirect();
        }

    }

    /**
     * Show attendance for editing.
     *
     * @param Request $request
     * @param Model   $attendance
     *
     * @return Response
     */
    public function edit(AttendanceRequest $request, Attendance $attendance)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('employee::attendance.name'))
            ->view('employee::attendance.edit', true)
            ->data(compact('attendance'))
            ->output();
    }

    /**
     * Update the attendance.
     *
     * @param Request $request
     * @param Model   $attendance
     *
     * @return Response
     */
    public function update(AttendanceRequest $request, Attendance $attendance)
    {
        try {
            $attributes = $request->all();

            $attendance->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('employee::attendance.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('employee/attendance/' . $attendance->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('employee/attendance/' . $attendance->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the attendance.
     *
     * @param Model   $attendance
     *
     * @return Response
     */
    public function destroy(AttendanceRequest $request, Attendance $attendance)
    {
        try {

            $attendance->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('employee::attendance.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('employee/attendance/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('employee/attendance/' . $attendance->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple attendance.
     *
     * @param Model   $attendance
     *
     * @return Response
     */
    public function delete(AttendanceRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('employee::attendance.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('employee/attendance'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/employee/attendance'))
                ->redirect();
        }

    }

    /**
     * Restore deleted attendances.
     *
     * @param Model   $attendance
     *
     * @return Response
     */
    public function restore(AttendanceRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('employee::attendance.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/employee/attendance'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/employee/attendance/'))
                ->redirect();
        }

    }

}
